/*
* Copyright (C) 2001-2008, CompHEP Collaboration
* Copyright (C) 1997, Alexander Pukhov 
* with changes by V.Ilyin for #-models, 6 Sept 2000
* ------------------------------------------------------
* $Id: sf_pdf.c,v 1.9 2003/07/01 14:29:38 kryukov Exp $
*
* $Log: sf_pdf.c,v $
* Revision 1.9  2003/07/01 14:29:38  kryukov
* Now list of pdf's display from 1 line
*
* Revision 1.8  2003/07/01 13:53:09  sherstnv
* QCD Lambda is taken from cteq** file now
* The strfun dir contains *.tbl - original cteq
* and *.pdf - our format of cteq.
*
* Revision 1.7  2003/06/25 18:46:41  kryukov
* *** empty log message ***
*
* Revision 1.6  2003/04/22 09:01:19  kryukov
* Apply indent to improve readability of files
*
* Revision 1.5  2003/04/19 18:32:30  kryukov
* Change format menu function. Now it must return int value (position of selection)
*
*/

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "chep_limits.h"
#include "unix_utils.h"
#include "files.h"
#include "syst.h"

#include "tag_reader.h"
#include "tag_parser.h"
#include "strfun_par.h"
#include "pdf.h"
#include "sf_pdf.h"

static shortstr pdfName[2] = {"PDF:OFF", "PDF:OFF"};
static shortstr pdfBeam[2] = {"parton", "parton"};

static pdfStr pdfData[9];       /*  #-mdl */

int p_pdf (char * p_name) {
  int val = 0;
  pdfList * list = NULL;
  comphepPdfList (p_name, &list);
  if (list) {
    delPdfList (list);
    val = 1;
  }
  return val;
}

void info_pdf (int i, Str_fun_Info * info) {
  int PDFid = 0;
  int PDFgr = 0;
  int unknown_pdf;

  strcpy (info->pdf_name, "CTEQ");
  info->prt_mass = 0.938;
  strcpy (info->prt_name, pdfBeam[i]);
  strcpy (info->version, pdfName[i] + 4 + 4); /* remove 'PDF:' and 'cteq' */

  unknown_pdf = CERNpdf_number (info->pdf_name, info->version, &PDFid, &PDFgr);
  if (unknown_pdf) {
    fprintf (stderr, "*** Uknown PDF %s or its version %s", info->pdf_name, info->version);
    exit (1);
  }

  info->N_extra_commands = 2;
  snprintf (info->extra_commands[0], 7 + intlen (PDFid), "PDFid=%i", PDFid);
  snprintf (info->extra_commands[1], 7 + intlen (PDFgr), "PDFgr=%i", PDFgr);
  info->PDFLIBset = PDFid;
  info->PDFLIBgroup = PDFgr;
  info->LHAPDFset = 0;
  info->LHAPDFmember = 0;
}

void n_pdf (int i, char * beam, char * pdf) {
  i--;
  if (strlen(pdfName[i])) {
    strcpy (pdf, pdfName[i]);
  } else {
    strcpy (pdfBeam[i], "parton");
    strcpy (pdf, "PDF:OFF");
  }
  strcpy (beam, pdfBeam[i]);
}

int r_pdf (int i, char * name) {
  int ret_code = 0;
  int len;

  --i;
  if (strstr (name, "PDF:")) {
    char * bname = strstr (name, "(");
    strcpy (pdfBeam[i], bname + 1);
    len = strlen (pdfBeam[i]);
    pdfBeam[i][len - 1] = '\0';
    trim(pdfBeam[i]);

    len = strlen (name) - len - 1;
    name[len] = '\0';
    strcpy (pdfName[i], name);
    trim(pdfName[i]);
    ret_code = 1;
  }

  return ret_code;
}


/* initialization function 
  i - beam number
  be - return value (???)
  mass - beam particle mass
  return the string number in PDF menu 
*/

int be_pdf (int i, double * be, double * mass, char * p_name) {
  pdfList *list = NULL;
  pdfList *list_;
  int ret_code = 0;
  static int first[2] = {1, 1};
  int ip, xip;                  /* #-mdl */
  char *xpar;                   /* #-mdl */
  xpar = "anti-proton";         /* #-mdl */
  shortstr tmpname;

  i--;
  if (!first[i])
    freePdfData (pdfData + i);

  comphepPdfList (p_name, &list);
  list_ = list;
  sprintf (tmpname, "%s(%s)", pdfName[i] + 4, pdfBeam[i]);
  while (list_ && ret_code == 0) {
    if (!strcmp (list_->name, tmpname)) {
      if (i == 0 || !pdfName[0])
        for (ip = 2; ip < 9; ip++){
          xip = ip;
          if (strstr (pdfBeam[i], xpar)) {
            if (ip == 8) xip = 4;
            if (ip == 4) xip = 8;
            if (ip == 7) xip = 5;
            if (ip == 5) xip = 7;
          }
          getPdfData (list_->pathfile, list_->file, xip, pdfData + ip);   /* #-mdl */
        }
      ret_code = !getPdfData (list_->pathfile, list_->file, list_->position, pdfData + i);
    }
    list_ = list_->next;
  }
  if (list) {
    delPdfList (list);
  }

  if (ret_code) {
    first[i] = 0;
    if (pdfData[i].alpha)
      set_alphaMode (i + 1);
    if (pdfData[i].pow1 < 0) {
      *be = pdfData[i].pow1 + 1;
    } else {
      *be = 1.;
    }
    *mass = pdfData[i].mass;
  }

  return ret_code;
}


/* return pdf value of i-th parton in a point (x,Q) */
/* stability interface to double interFunc (double x, double q, pdfStr * W) from pdf.c */
double c_pdf (int i, double x, double q) {
  double r;

  i--;
  r = interFunc (x, q, pdfData + i);
  if (!r) {
    return 0.;
  }

  if (pdfData[i].pow0) {
    r *= pow (x, pdfData[i].pow0);
  }

  if (pdfData[i].pow1 > 0) {
    r *= pow (1 - x, pdfData[i].pow1);
  }

  if (r < 0) {
    r = 0;
  }

  return r;
}

double alpha_pdf (double q) {
  return interAlpha (q, pdfData + get_alphaMode() - 1);
}

#ifndef LHAPDF
int pdfnamecmp (void) {
  if (!strcmp (pdfName[0], pdfName[1]))
    return 1;
  return 0;
}
#endif
