/*
* Copyright (C) 2008, CompHEP Collaboration
* Author: Alexander Sherstnev 
* ------------------------------------------------------
* $Id$
*
* $Log$
*/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "chep_limits.h"
#include "files.h"
#include "drandXX.h"
#include "lhapdf.h"
#include "lhef_routines.h"
#include "LesHouches.h"
#include "evnt_tools.h"
#include "tag_reader.h"
#include "tag_parser.h"
#include "event_reader.h"

static char opdfname[2048];
static int opdfset, opdfmem;
static int kf1, kf2;
static int nprup = 0;
static int idwtup;
static double e1, e2;
static double * cs;
static double * cserr;

static int read_config (FILE * f)
{
  char buff[2048];
  int err;

  if (NULL == fgets (buff, 2048, f)) return -1;
  if (NULL == fgets (buff, 2048, f)) return -1;
  seedXX (buff);

  if (NULL == fgets (buff, 2048, f)) return -1;
  if (NULL == fgets (buff, 2048, f)) return -1;
  err = sscanf (buff, "%i", &opdfset);
  if (1 != err)  return -1;

  if (NULL == fgets (buff, 2048, f)) return -1;
  if (NULL == fgets (buff, 2048, f)) return -1;
  err = sscanf (buff, "%i", &opdfmem);
  if (1 != err)  return -1;

  return 0;
}


/*********************************************************************************************/
#ifdef LHAPDF
static int init_lhapdf (int bnum, int set, int mem, int kfcode) {
  longstr pathtoindexfile;
  longstr tmppath;
  int status = setLHAPDFIndexpath (pathtoindexfile);
  int undefined = 1;
  lhapdfList * list = NULL;
  lhapdfList * list_;

  if (!status) {
    fprintf (stderr, "Warning! Can not find PDFsets.index\n");
    return 0;
  }
  sprintf (tmppath, "%s/share/lhapdf/PDFsets/", pathtolhapdf);
  comphepLhapdfList (tmppath, pathtoindexfile, &list);
  list_ = list;

  status = 0;
  while (list_) {
    if (set == list_->set && mem == list_->mem) {
      undefined = 0;
      initLHAPDF (bnum, tmppath, list_->name, set, mem, kfcode);
      status = 1;
      break;
    }
    list_ = list_->next;
  }
  if (list) {
    delLhapdfList (list);
  }
  return status;
}

int write_event_cap_lhef_cpyth (char fname[], char mode[]) {
  int idwtup = 3;
  long pos;
  int i;

  FILE * outFile = fopen (fname, mode);
  if (!outFile) {
    return 0;
  }

  fprintf (outFile, "<LesHouchesEvents version=\"1.0\">\n");
  fprintf (outFile, "<!-- File generated with CompHEP 4.5.0 -->\n");
  fprintf (outFile, "<!-- \n");
  fprintf (outFile, "     Preliminary version, it is compatible with the Les Houches event file\n");
  fprintf (outFile, "     format (hep-ph/0609017), but contains extra tags.\n");
  fprintf (outFile, "-->\n");
  fprintf (outFile, "<header>\n");

/*  fprintf (outFile, "%s", prepare_hepml_header ());*/
  fprintf (outFile, "</header>\n");
  fprintf (outFile, "<init>\n");
  fprintf (outFile, "%i %i %17.10E %17.10E %i %i %i %i %i %i\n",
                    kf1, 
                    kf2, 
                    e1, 
                    e2, 
                    opdfset, 
                    opdfset, 
                    opdfmem, 
                    opdfmem, 
                    idwtup, 
                    nprup);
  for (i = 0; i < nprup; ++i) {
  fprintf (outFile, "%17.10E %17.10E %17.10E %i\n", cs[i], cserr[i], 1.0, i);
  }
  fprintf (outFile, "</init>\n");

  pos = ftell (outFile);
  fclose (outFile);

  return pos;
}

static int reweighter (char * config, char * source, char * target) 
{
  int i, num;
  double * pdf_ratio;
  int * epos;
  int * inumcount;
  int * onumcount;
  char * stop = NULL;
  char buff[2048];
  FILE * s = fopen (source, "r");
  FILE * t;
  int set1, set2;
  int mem1, mem2;
  int iniposition_s;
  int iniposition_ss;
  int iniposition_t;
  long pos;
  eventUP evnt;
  double pdfmax = 0.;

  while (!stop) {
    fgets (buff, 2048, s);
    if (feof (s)) return -4;
    stop = strstr (buff, "<init>");
  }
  fgets (buff, 2048, s);
  if (10 != sscanf (buff, " %d %d %le %le %d %d %d %d %d %d", &kf1, &kf2, &e1, &e2, &set1, &set2, &mem1, &mem2, &idwtup, &nprup)) {
    return -4;
  }

  if (4 == set1) {
      switch (mem1) {
        case 34: {set1 = 19150; mem1 = 0; break;} /* cteq4m  */
        case 32: {set1 = 19170; mem1 = 0; break;} /* cteq4l  */
        case 53: {set1 = 19051; mem1 = 0; break;} /* cteq5m1 */
        case 48: {set1 = 19050; mem1 = 0; break;} /* cteq5m  */
        case 46: {set1 = 19070; mem1 = 0; break;} /* cteq5l  */
        case 55: {set1 = 10041; mem1 = 0; break;} /* cteq6l  */
/*        case 56: {set1 = ?????; mem1 = 0; break;}  cteq6d  */
        case 57: {set1 = 10050; mem1 = 0; break;} /* cteq6m  */
        case 58: {set1 = 10042; mem1 = 0; break;} /* cteq6l1 */
        default: {
          fprintf(stderr,"pdf_reweighter: error! can't detect PDF set/mem numbers in the event file (set = %i, mem = %i).\n", set1, mem1);
          return -1;
        }
      }
  }
  if (4 == set2) {
      switch (mem1) {
        case 34: {set2 = 19150; mem2 = 0; break;} /* cteq4m  */
        case 32: {set2 = 19170; mem2 = 0; break;} /* cteq4l  */
        case 53: {set2 = 19051; mem2 = 0; break;} /* cteq5m1 */
        case 48: {set2 = 19050; mem2 = 0; break;} /* cteq5m  */
        case 46: {set2 = 19070; mem2 = 0; break;} /* cteq5l  */
        case 55: {set2 = 10041; mem2 = 0; break;} /* cteq6l  */
/*        case 56: {set2 = ?????; mem2 = 0; break;}  cteq6d  */
        case 57: {set2 = 10050; mem2 = 0; break;} /* cteq6m  */
        case 58: {set2 = 10042; mem2 = 0; break;} /* cteq6l1 */
        default: {
          fprintf(stderr,"pdf_reweighter: error! can't detect PDF set/mem numbers in the event file (set = %i, mem = %i).\n", set1, mem1);
          return -1;
        }
      }
  }

  cs    = malloc (nprup * sizeof (double));
  cserr = malloc (nprup * sizeof (double));
  inumcount = malloc (nprup * sizeof (double));
  onumcount = malloc (nprup * sizeof (double));

  for (i = 0; i < nprup; ++i) {
  double wght;
  int nnum;
    fgets (buff, 2048, s);
    if (4 != sscanf (buff, " %le %le %le %i", &cs[i], &cserr[i], &wght, &nnum)) {
      return -4;
    }
    inumcount[i] = 0;
    onumcount[i] = 0;
  }

  stop = NULL;
  while (!stop) {
    fgets (buff, 2048, s);
    if (feof (s)) return -4;
    stop = strstr (buff, "</init>");
  }
  iniposition_ss = ftell (s);

  stop = NULL;
  while (!stop) {
    iniposition_s = ftell (s);
    fgets (buff, 2048, s);
    if (feof (s)) return -4;
    stop = strstr (buff, "<event>");
  }

  num = 0;
  pos = iniposition_s;
  while (!testEvent (s, source, pos)) {
    ++num;
    pos = ftell (s);
  }
  pdf_ratio = malloc ((num + 1) * sizeof (double));
  epos      = malloc ((num + 1) * sizeof (int));


  if (0 == init_lhapdf (0, set1, mem1, kf1) || 0 == init_lhapdf (1, set2, mem2, kf2)) {
    return -1;
  }

  num = 0;
  epos[0] = iniposition_s;
  while (0 == getLHAevent (source, s, epos[num], &evnt)) {
    ++num;
    epos[num] = ftell (s);
    pdf_ratio[num] = lhapdfValCPYTH ( evnt.momentumUP[2][0] / e1, evnt.QscaleUP, evnt.IDpartUP[0]) * 
                     lhapdfValCPYTH (-evnt.momentumUP[2][1] / e2, evnt.QscaleUP, evnt.IDpartUP[1]);
   ++inumcount[evnt.IDprocUP - 1];
  }

  if (0 == init_lhapdf (1, opdfset, opdfmem, kf1) || 0 == init_lhapdf (1, opdfset, opdfmem, kf2)) {
    return -1;
  }

  num = 0;
  while (0 == getLHAevent (source, s, epos[num], &evnt)) {
    ++num;
    pdf_ratio[num] /= lhapdfValCPYTH ( evnt.momentumUP[2][0] / e1, evnt.QscaleUP, evnt.IDpartUP[0]) * 
                      lhapdfValCPYTH (-evnt.momentumUP[2][1] / e2, evnt.QscaleUP, evnt.IDpartUP[1]);
  }

  ++num;
  for (i = 1; i < num; ++i) {
    if (pdfmax < pdf_ratio[i]) pdfmax = pdf_ratio[i];
  }

  stop = NULL;
  write_event_cap_lhef_cpyth (target, "w");
  t = fopen (target, "a+");
  fseek (s, iniposition_ss, SEEK_SET);
  while (!stop) {
    iniposition_t = ftell (t);
    fgets (buff, 2048, s);
    if (feof (s)) return -4;
    stop = strstr (buff, "<event>");
    if (!stop) fputs (buff,t);
  }

  pos = ftell (t);
  for (i = 1; i < num; ++i) {
    double ddd = drandXX ();
    if (ddd < pdf_ratio[i] / pdfmax) {
      moveEvent (-1, source, s, epos[i - 1], t, pos);
      pos = ftell (t);
    }
  }
  fclose (s);

  fseek (t, 0L, SEEK_SET);
  stop = NULL;
  while (!stop) {
    iniposition_t = ftell (t);
    fgets (buff, 2048, t);
    if (feof (t)) return -4;
    stop = strstr (buff, "<event>");
  }
  pos = iniposition_t;
  while (0 == getLHAevent (target, t, pos, &evnt)) {
    pos = ftell (t);
   ++onumcount[evnt.IDprocUP - 1];
  }
  fclose (t);
  for (i = 0; i < nprup; ++i) {
    double k = (double)onumcount[i] / (double)inumcount[i];
    cs[i] *= k;
    cserr[i] *= k;
  }
  write_event_cap_lhef_cpyth (target, "r+");

  free (epos);
  free (pdf_ratio);
  free (cs);
  free (cserr);
  free (inumcount);
  free (onumcount);

  return 0;
}
#endif


int main (int argc, char** argv)
{
  int i;
  int frmt;
  char target[2048];
  char source[2048];
  char config[2048];
  FILE * iniFile;
  FILE * outFile;
  FILE * conFile;

  strcpy (target, "Reweighted.PEV");
  strcpy (source, "events.PEV");
  strcpy (config, "config.reweight");

  for (i = 1; i < argc; ++i) {
    if (!strcmp (argv[i], "-o")) {
      strcpy (target, argv[++i]);
    }
    if (!strcmp (argv[i], "-i")) {
      strcpy (source, argv[++i]);
    }
    if (!strcmp (argv[i], "-c")) {
      strcpy (config, argv[++i]);
    }
  }

  iniFile = fopen (source, "r");
  if (!iniFile) {
    fprintf(stderr,"pdf_reweighter: error! Initial event file not found: %s\n", source);
    return -1;
  }
  frmt = CheckFormat (iniFile);
  if (4 != frmt) {
    fprintf(stderr, "pdf_reweighter: error! the program supports LHE format only.\n");
    return -5;
  }
  fclose (iniFile);

  outFile = fopen (target, "r");
  if (outFile) {
    fprintf(stderr,"pdf_reweighter: error! Final event file found: %s\n", target);
    fclose (outFile);
    return -2;
  }

  conFile = fopen (config, "r");
  if (!conFile) {
    fprintf(stderr,"pdf_reweighter: error! File not found: %s\n", config);
    return -3;
  } else {
    if (read_config (conFile)) {
      fprintf(stderr,"pdf_reweighter: error! strange config file.\n");
      return -4;
    }
  }
  fclose (conFile);

#ifdef LHAPDF
  {
    midstr _pathtolhapdf;
    char * p = getenv ("LHAPDFPATH");
    if (!p) {
      fprintf (stderr, " Environment variable LHAPDFPATH is not defined.\n");
      exit (-2);
    }
    strcpy (_pathtolhapdf, p);
    sprintf (pathtolhapdf, "%s/", _pathtolhapdf);
  }
  reweighter (config, source, target);
#else
  fprintf (stderr, "pdf_reweighter: warning! The program can work with LHAPDF only, exit.\n");

#endif

  return 0;
}
